﻿Imports Microsoft.Practices.Prism.ViewModel
Imports System.Text.RegularExpressions
Imports System.Collections.ObjectModel

Public Class MainWindowViewModel
    Inherits NotificationObject

    Public Event OnSnippetTypeListChanged()

    Public Sub New()
        InitCommand()
    End Sub

#Region "   数据属性"

    '
    'SnippetList As List(Of SnippetInfo)
    '
    Private mSnippetList As List(Of SnippetInfo)
    Public Property SnippetList() As List(Of SnippetInfo)
        Get
            Return mSnippetList
        End Get
        Set(ByVal Value As List(Of SnippetInfo))
            mSnippetList = Value
            RaisePropertyChanged("SnippetList")
        End Set
    End Property

    '
    'SnippetTypeList As List(Of String)
    '
    Private mSnippetTypeList As New ObservableCollection(Of String)
    Public Property SnippetTypeList() As ObservableCollection(Of String)
        Get
            Return mSnippetTypeList
        End Get
        Set(ByVal Value As ObservableCollection(Of String))
            mSnippetTypeList = Value
            RaisePropertyChanged("SnippetTypeList")
        End Set
    End Property

    '
    'SelectedSnippet As SnippetInfo
    '
    Private mSelectedSnippet As SnippetInfo
    Public Property SelectedSnippet() As SnippetInfo
        Get

            Return mSelectedSnippet
        End Get
        Set(ByVal Value As SnippetInfo)
            mSelectedSnippet = Value
            RefrashTypeList()
            RaisePropertyChanged("SelectedSnippet")
        End Set
    End Property

    '
    'IsEdit As Boolean
    '
    Private mIsEdit As Boolean = False
    Public Property IsEdit() As Boolean
        Get
            Return mIsEdit
        End Get
        Set(ByVal Value As Boolean)
            mIsEdit = Value
            RaisePropertyChanged("IsEdit")
        End Set
    End Property

#End Region

#Region "   命令属性"
    Public Sub InitCommand()
        XmlDataService.LoadSnippets()
        SnippetList = gSnippets.SnippetList
        RefrashTypeList()
        NewCommand()
    End Sub

    Public Sub NewCommand()
        SelectedSnippet = New SnippetInfo
        IsEdit = False
    End Sub

    Public Sub SaveCommand()
        If IsEdit Then

        Else
            SnippetList.Add(SelectedSnippet)
            IsEdit = True
        End If
        XmlDataService.SaveSnippets()

        Refresh()
    End Sub

    Public Sub DeleteCommand()
        SnippetList.Remove(SelectedSnippet)
        XmlDataService.SaveSnippets()
        Refresh()
    End Sub

    Public Sub SelectionChangedCommand(ByVal item As SnippetInfo)
        If item Is Nothing Then Exit Sub
        SelectedSnippet = item
        IsEdit = True
    End Sub

    Public Sub WriteCommand()
        Dim tds As New TextDataService
        tds.CreateTempSnippet(SelectedSnippet)
    End Sub

    Public Sub SearchFullCommand(ByVal searchStr As String, ByVal searchType As String)
        SnippetList = gSnippets.SnippetList.FindAll( _
            New Predicate(Of SnippetInfo)( _
                Function(s As SnippetInfo) _
                    s.Name.ToLower = searchStr.ToLower And s.SnippetType = searchType))
    End Sub

    Public Sub SearchCommand(ByVal searchStr As String, ByVal searchType As String)
        SnippetList = gSnippets.SnippetList.FindAll( _
            New Predicate(Of SnippetInfo)( _
                Function(s As SnippetInfo) _
                    (s.Name.ToLower.Contains(searchStr.ToLower) _
                    Or s.Comment.ToLower.Contains(searchStr.ToLower)) And s.SnippetType = searchType))

    End Sub

    Public Sub ReplaceAllCommand(ByVal param As String)
        Dim s As String = SelectedSnippet.SnippetText
        Dim s1 As String = param
        s = Regex.Replace(s, "@" + s1 + "@", "#!!!#")
        s = Regex.Replace(s, s1, "@" + s1 + "@")
        SelectedSnippet.SnippetText = Regex.Replace(s, "#!!!#", "@" + s1 + "@")

    End Sub

    Public Sub DeleteAllCommand(ByVal param As String)
        SelectedSnippet.SnippetText = SelectedSnippet.SnippetText.Replace("@" + param + "@", param)
        SelectedSnippet.Params = SelectedSnippet.Params.Replace(" " + param + " ,", "")
    End Sub

    Public Sub ReplaceCommand(ByVal param As String, ByVal pos As Integer, ByVal len As Integer)
        SelectedSnippet.SnippetText = SelectedSnippet.SnippetText.Remove(pos, len)
        SelectedSnippet.SnippetText = SelectedSnippet.SnippetText.Insert(pos, "@" + param + "@")
        GetParam()
    End Sub

    Public Sub RenameCommand(ByVal param As String, ByVal newParam As String)
        If SelectedSnippet.Params.Contains(newParam) Then
            MsgBox("在参数中存在或者部分存在新参数", vbOKOnly, "通知")
            Exit Sub
        End If

        Dim strArr() As String = SelectedSnippet.Params.Split(",".ToCharArray)
        For Each s As String In strArr

            If newParam.Contains(s.Trim) And s.Trim.Length > 0 Then
                MsgBox("在新参数中存在或者部分存在现有参数", vbOKOnly, "通知")
                Exit Sub
            End If
        Next

        SelectedSnippet.SnippetText = SelectedSnippet.SnippetText.Replace("@" + param.Trim + "@", "@" + newParam + "@")
        SelectedSnippet.Params = SelectedSnippet.Params.Replace(param, " " + newParam + " ")
    End Sub

    Public Sub SnippetTextChangedCommand(ByVal s As String)
        mSelectedSnippet.SnippetText = s
    End Sub

#End Region

#Region "   业务逻辑"
    Private Sub Refresh()
        SnippetList = Nothing
        SnippetList = gSnippets.SnippetList
        RefrashTypeList()
        NewCommand()
    End Sub

    Public Sub RefrashTypeList()
        Dim newList = New ObservableCollection(Of String)
        newList.Add("")
        For Each s As SnippetInfo In SnippetList
            If Not newList.Contains(s.SnippetType) Then
                newList.Add(s.SnippetType)
            End If
        Next
        SnippetTypeList = newList
        RaiseEvent OnSnippetTypeListChanged()
    End Sub

    Private KeyWords() As String =
            {
                "AddHandler", "AddressOf", "AndAlso", "Alias", "And", "Ansi", "As", "Assembly", "Auto", "Boolean",
                "ByRef", "Byte", "ByVal", "Call", "Case", "Catch", "CBool", "CByte", "CChar",
                "CDate", "CDec", "CDbl", "Char", "CInt", "Class", "CLng", "CObj", "Const",
                "CShort", "CSng", "CStr", "CType", "Date", "Decimal", "Declare", "Default",
                "Delegate", "Dim", "DirectCast", "Do", "Double", "Each", "Else", "ElseIf", "End",
                "Enum", "Erase", "Error", "Event", "Exit", "False",
                "Finally", "For", "Friend", "Function", "Get", "GetType", "GoTo", "Handles", "If",
                "Implements", "Imports", "In", "Inherits", "Integer", "Interface",
                "Is", "Let", "Lib", "Like", "Long", "Loop", "Me", "Mod", "Module",
                "MustInherit", "MustOverride", "MyBase", "MyClass", "Namespace", "New", "Next", "Not", "Nothing",
                "NotInheritable", "NotOverridable", "Object", "On", "Option", "Optional", "Or", "OrElse",
                "Overloads", "Overridable", "Overrides", "ParamArray", "Preserve", "Private", "Property", "Protected", "Public",
                "RaiseEvent", "ReadOnly", "ReDim", "RemoveHandler", "Resume", "Return",
                "Select", "Set", "Shadows", "Shared", "Short", "Single", "Static", "Step", "Stop",
                "String", "Structure", "Sub", "SyncLock", "Then", "Throw",
                "To", "True", "Try", "TypeOf", "Unicode", "Until", "Variant", "When", "While",
                "With", "WithEvents", "WriteOnly", "Xor", "#", "Region"
        }

    Public Function GetBlueKeyWords() As IEnumerable(Of String)
        Return KeyWords
    End Function

    Private Sub GetParam()
        Dim ms As MatchCollection = Regex.Matches(SelectedSnippet.SnippetText, "@\w+@", RegexOptions.Compiled Or RegexOptions.IgnorePatternWhitespace)

        SelectedSnippet.Params = ""
        For Each m As Match In ms
            If Not SelectedSnippet.Params.Contains(m.ToString.Replace("@", " ")) Then
                SelectedSnippet.Params += m.ToString.Replace("@", " ") + ","
            End If
        Next
    End Sub
#End Region



End Class
