﻿Imports System.Text.RegularExpressions
Imports CodeBoxControl.Decorations

Class MainWindow
    Dim WithEvents vm As MainWindowViewModel

    Public Sub New()

        ' 此调用是设计器所必需的。
        InitializeComponent()

        ' 在 InitializeComponent() 调用之后添加任何初始化。
        vm = New MainWindowViewModel
        Me.DataContext = vm
        InitCodeBox()
        
    End Sub

#Region "   命令发布"

    Private Sub btnNew_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnNew.Click
        vm.NewCommand()
    End Sub

    Private Sub btnSave_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnSave.Click
        btnDelete.Focus()
        vm.SaveCommand()
        MsgBox("保存成功")
    End Sub

    Private Sub btnDelete_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnDelete.Click
        vm.DeleteCommand()
    End Sub

    Private Sub btnReplaceAll_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnReplaceAll.Click
        vm.ReplaceAllCommand(tbParams.SelectedText.Trim)
    End Sub

    Private Sub btnDeleteAll_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnDeleteAll.Click
        vm.DeleteAllCommand(tbParams.SelectedText.Trim)
    End Sub

    Private Sub btnReplace_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnReplace.Click
        vm.ReplaceCommand(tbCode.SelectedText, tbCode.SelectionStart, tbCode.SelectionLength)
    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles Button1.Click
        vm.WriteCommand()
    End Sub

    Private Sub tbSearch_TextChanged(ByVal sender As System.Object, ByVal e As System.Windows.Controls.TextChangedEventArgs) Handles tbSearch.TextChanged

        If tbSearch.Text.Contains(";") Then
            vm.SearchFullCommand(tbSearch.Text.Replace(";", ""), cbSearchType.Text)
        Else
            vm.SearchCommand(tbSearch.Text, cbSearchType.Text)
        End If
    End Sub

    Private Sub cbSearchType_SelectionChanged(ByVal sender As System.Object, ByVal e As System.Windows.Controls.SelectionChangedEventArgs)
        If tbSearch.Text.Contains(";") Then
            vm.SearchFullCommand(tbSearch.Text.Replace(";", ""), cbSearchType.SelectedValue)
        Else
            vm.SearchCommand(tbSearch.Text, cbSearchType.SelectedValue)
        End If
    End Sub

    Private Sub dg_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Input.MouseButtonEventArgs) Handles dg.MouseUp
        If dg.SelectedItems.Count > 0 Then
            vm.SelectionChangedCommand(CType(dg.SelectedItem, SnippetInfo))
        End If
    End Sub

    Private Sub btnRename_Click(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnRename.Click
        Dim newParam As String = InputBox("请输入新的标签名", "新的标签")
        If newParam.Length = 0 Then Exit Sub
        vm.RenameCommand(ParamSelection, newParam)
    End Sub
#End Region

#Region "   窗口逻辑"

    '与界面关系太紧密而成为窗口逻辑
    Private ParamSelection As String = ""

    Private Sub tbParams_PreviewMouseLeftButtonUp(ByVal sender As Object, ByVal e As System.Windows.Input.MouseButtonEventArgs) Handles tbParams.PreviewMouseLeftButtonUp
        Dim ss() As String = tbParams.Text.Split(",".ToCharArray)
        Dim startPos As Integer = 0
        Dim endPos As Integer = 0
        For Each s As String In ss

            startPos = endPos
            endPos += s.Length + 1
            If startPos <= tbParams.SelectionStart And endPos >= tbParams.SelectionStart Then
                tbParams.SelectionStart = startPos
                tbParams.SelectionLength = endPos - startPos - 1
                Exit Sub
            End If
        Next
    End Sub

    Private Sub tbParams_SelectionChanged(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles tbParams.SelectionChanged
        btnDeleteAll.IsEnabled = (tbParams.SelectionLength > 0)
        btnReplaceAll.IsEnabled = (tbParams.SelectionLength > 0)
        btnRename.IsEnabled = (tbParams.SelectionLength > 0)
        ParamSelection = tbParams.SelectedText
    End Sub

    Private Sub tbCode_SelectionChanged(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles tbCode.SelectionChanged
        btnReplace.IsEnabled = (tbCode.SelectionLength > 0)
    End Sub

    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles Button2.Click
        cbSearchType.Text = ""
        tbSearch.Text = ""
    End Sub

    Private Sub InitCodeBox()
        tbCode.Decorations.Clear()
        'Color most reserved words blue
        Dim blueKeyWords As New MultiRegexWordDecoration
        blueKeyWords.Brush = New SolidColorBrush(Colors.Blue)
        blueKeyWords.Words.AddRange(vm.GetBlueKeyWords())
        tbCode.Decorations.Add(blueKeyWords)

        Dim YinhaoWords As New RegexDecoration
        YinhaoWords.Brush = Brushes.IndianRed
        YinhaoWords.RegexString = """.+?"""
        tbCode.Decorations.Add(YinhaoWords)

        'Color Comment Words
        Dim CommentWords As New RegexDecoration
        CommentWords.Brush = Brushes.Green
        CommentWords.RegexString = "'.*"
        tbCode.Decorations.Add(CommentWords)

        'Color Params Words
        Dim ParamsWordsFore As New RegexDecoration
        ParamsWordsFore.DecorationType = EDecorationType.Hilight
        ParamsWordsFore.Brush = Brushes.WhiteSmoke
        ParamsWordsFore.RegexString = "@.+?@"
        tbCode.Decorations.Add(ParamsWordsFore)

    End Sub

    Private Sub MainWindow_Loaded(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles Me.Loaded
        Me.Left = SystemParameters.WorkArea.Width - Me.Width
        Me.Top = SystemParameters.WorkArea.Height - Me.Height
    End Sub
#End Region

    
    Private Sub tbPath_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        gSnippets.SnippetPath = InputBox("请输入我的代码段文件夹。", DefaultResponse:="d:\Users\alan\Documents\Visual Studio 2010\Code Snippets\Visual Basic\My Code Snippets") + "\tempSnippet.snippt"
    End Sub
End Class
